@@ -38,13 +38,16 @@ class HGDL:
     grad : Callable
         The gradient of the function to be MINIMIZED. A callable that accepts an np.ndarray and optional arguments, and returns a vector
         (np.ndarray) of shape (D), where D is the dimensionality of the space in which the
-        optimization takes place.
+        optimization takes place. Remeber that D changes if constraints are used; account for the derivatives with respect to multipliers and
+        slack variables.
     bounds : np.ndarray
-        The bounds in which to optimize; an np.ndarray of shape (D x 2), where D is the dimensionality of the space in which the
-        optimization takes place.
+        The bounds of the domain; an np.ndarray of shape (D x 2), where D is the dimensionality of the space in which the
+        optimization takes place. Here D is the dimension of the original input space without slack variables or multipliers. Those bounds
+        can be specified when seeting upt he constraint.
     hess : Callable, optional
         The Hessian of the function to be MINIMIZED. A callable that accepts an np.ndarray and optional arguments, and returns a
-        np.ndarray of shape (D x D). The default value is no-op.
+        np.ndarray of shape (D x D). The default value is no-op. If provided the Hessian should also include the derivatives with respect to
+        slack variables and multipliers, if constraints are used.
     num_epochs : int, optional
         The number of epochs the algorithm runs through before being terminated. One epoch is the convergence of all local walkers,
         the deflation of the identified optima, and the global replacement of the walkers. Note, the algorithm is running asynchronously, so a high number
@@ -58,19 +61,22 @@ class HGDL:
         individuals that should be returned. The callable should return the positions of the offspring
         individuals as an np.ndarray of shape (number_of_offspring x D).
     local_optimizer : Callable or str, optional
-        The local optimizer that is used for the local-walker optimization. The options are
-        `dNewton` (the default Newton method that need a Hessian), `L-BFGS-B`, `BFGS`, `CG`, `Newton-CG` and most other scipy.optimize.minimize
-        local optimizers. The above methods have been tested, but most others should work. Visit the `scipy.optimize.minimize` docs for specifications
+        The local optimizer that is used. The options are
+        `dNewton` (default), `L-BFGS-B`, `BFGS`, `CG`, `Newton-CG`.
+        The above methods have been tested, but most others should work. Visit the `scipy.optimize.minimize` docs for specifications
         and limitations of the local methods. The parameter also accepts a callable that accepts as input a function, gradient, Hessian,
-        bounds (all as specified above), and args, and returns an object similar to the scipy.optimize.minimize methods.
+        bounds (all as specified above), and args, and returns an object similar to the scipy.optimize.minimize methods. `DNewton` should be used
+        for constrained optimization.
     number_of_optima : int, optional
         The number of optima that will be stored in the optima list and deflated. The default is 1e6.
+        After that number is reached, worse-performing optima will not be stored or deflated.
     radius: float, optional
         The radius of the deflation operator. The default is estimated from the size of the domain.
         This will be changed in future releases to be estimated from the curvature of the function
         at the optima.
     local_max_iter : int, optional
-        The number of iterations before local optimizations are terminated. The default is 100.
+        The number of iterations before local optimizations are terminated. The default is 1000.
+        It can be lowered when second-order local optimizaers are used.
     args : tuple, optional
         A tuple of arguments that will be communicated to the function, the gradient, and the Hessian callables.
         Default = ().
@@ -78,14 +84,15 @@ class HGDL:
         An optional n-tuple of constraint objects. See hgdl.constraints.NonLinearConstraints.
         The default is no constraints (). Make sure you use a `local_optimizer = "dNewton"` if constraints are used.
         Also, try to provide a Hessian callable. If not, the Hessian will be approximated from the gradient which is fine as long as
-        your gradient function is fast and exact. When constraints are used, the space is extended by a linear space with elements 'k' which
+        your gradient function is fast and exact. Don't bother with a constraint Hessian if you don't have the objective function Hessian; it won't be used.
+        When constraints are used, the space is extended by a linear space with elements 'k' which
         are the Lagrangian multipliers and slack variables and will be part of the result object.
 
     Attributes
     ----------
     optima : object
         Contains the attribute optima.list in which the optima are stored. 
-        However, the method 'get_latest(n)' should be used to access the optima.
+        However, the method 'get_latest()' should be used to access the optima.
 
 
     """
@@ -101,7 +108,7 @@ class HGDL:
 
         self.constr = constraints
         self.dim_x = len(bounds)
-        self.func = func #self.lagrangian ##this canalways happen
+        self.func = func
         self.grad = grad
         self.hess = hess
         self.L = self.lagrangian
@@ -109,7 +116,7 @@ class HGDL:
         self.Lhess = self.lagrangian_hess
         index = self.dim_x
         if self.constr and local_optimizer != "dNewton": raise Exception("Please use ``local_optimizer = 'dNewton' '' if constraints are used.")
-        if local_optimizer == "dNewton": print("Warning: dNewton will not adhere to your bounds. It is recommended to formulated your objective functions bounds-free by simple non-linear transformations.")
+        if local_optimizer == "dNewton": print("Warning: dNewton will not adhere to bounds. It is recommended to formulate your objective function such that it is defined on R^N by simple non-linear transformations.")
         for c in self.constr:
             if c.ctype == "=":
                 c.set_multiplier_index(index)
@@ -164,7 +171,6 @@ class HGDL:
         client = self._init_dask_client(dask_client)
         self.tolerance = tolerance
         logger.debug(client)
-        if x0 is not None and len(x0[0]) != self.dim: raise Exception("The given starting locations do not have the right dimensionality.")
         self.x0 = self._prepare_starting_positions(x0)
         logger.debug("HGDL starts with: {}", self.x0)
         self.meta_data = meta_data(self)
@@ -180,6 +186,7 @@ class HGDL:
     def get_latest(self):
         """
         Function to request the current result.
+        No inputs
         """
         try:
             data, frames = self.transfer_data.get()
@@ -196,6 +203,7 @@ class HGDL:
         Function to request the final result.
         CAUTION: This function will block the main thread until
         the end of all epochs is reached.
+        No inputs.
         """
         try:
             self.optima = self.main_future.result()
@@ -226,10 +234,7 @@ class HGDL:
         res = self.get_latest()
         try:
             self.break_condition.set(True)
-            #self.client.gather(self.main_future)
             self.client.cancel(self.main_future)
-            #del self.main_future
-            #self.client.shutdown()
             self.client.close()
             logger.debug("HGDL kill client successful")
         except Exception as err:
@@ -239,6 +244,7 @@ class HGDL:
     ############USER FUNCTIONS END#############################################
     ###########################################################################
     def _prepare_starting_positions(self,x0):
+        if x0 is not None and len(x0[0]) != self.dim: x0 = misc.random_population(self.bounds,self.number_of_walkers)
         if x0 is None: x0 = misc.random_population(self.bounds,self.number_of_walkers)
         elif x0.ndim == 1: x0 = np.array([x0])
 
