@@ -37,6 +37,8 @@ def run_local_optimizer(d,x0,x_defl = []):
     tasks = []
     bf = client.scatter(d,workers = d.workers["walkers"])
     for i in range(min(len(x0),number_of_walkers)):
+        logger.debug("Worker ",i," submitted")
+        #print("Worker ",i," submitted", flush = True)
         worker = d.workers["walkers"][(int(i - ((i // number_of_walkers) * number_of_walkers)))]
         data = {"d":bf,"x0":x0[i],"x_defl":x_defl}
         tasks.append(client.submit(local_method,data,workers = worker))
